/* gmcalabel.c */

#include <string.h>
#include <math.h>
#include <gmcal/gmcalbasic.h>
#include <gmcal/gmcalspline.h>
#include <gmcal/gmcalabel.h>

static gint gmcal_create_abel_matrix(gdouble ***matrix, int n, gdouble *radii);
static gint gmcal_get_middle(gfloat **Z, gint dimX, gint dimY);
static void gmcal_abel_transform_vector(gdouble **matrix, gdouble *f, gdouble *h, gint n);
static void gmcal_abel_back_distribute_vector(gdouble **matrix, gdouble *f, gdouble *h, gint n, gdouble *radii);

static gint gmcal_get_middle(gfloat **Z, gint dimX, gint dimY)
{
  int i,j;
  float midpoint;
  float xm;
  float M;
  float threshold;
  float minZ, maxZ;

  minZ=maxZ=Z[0][0];

  for (i=0; i<dimY; i++) {
    for (j=0; j<dimX; j++) {
      maxZ=MAX(Z[i][j],maxZ);
      minZ=MIN(Z[i][j],minZ);
    }
  }

  threshold=maxZ-(maxZ-minZ)*1./3.;

  xm=0;
  M=0;
  for (i=0; i<dimY; i++) {
    for (j=0; j<dimX; j++) {
      if (Z[i][j]>threshold) {
        xm+=i*Z[i][j];
        M+=Z[i][j];
      }
    }
  }

  if (!M) return -1;
  
  midpoint=xm/M;

  printf("Mitte= %g\n",midpoint);
  return (int) (midpoint+0.5);
}

gint	gmcal_abel_create_symmetrical_copy	(gfloat *oldX,
						gfloat *oldY,
						gfloat **oldZ,
						gint olddimX,
						gint olddimY,
						gfloat **newX,
						gfloat **newY,
						gfloat ***newZ,
						gint *newdimX,
						gint *newdimY,
						gint not_used_yet)
{
  gint error;
  gint i;
  gint j;
  gint middle;
  gint radius;
  
  if ((error=gmcal_basic_matrix_check_integrity(oldX, oldY, oldZ, olddimX, olddimY))) {
    return error;
  }
  
  middle=gmcal_get_middle(oldZ, olddimX, olddimY);
  if (middle==-1) {
    return GMCAL_ABEL_MIDDLE_UNDETERMINED;
  }

  radius=MIN(middle, olddimY-middle-1);

  newdimY[0]=radius*2;
  newdimX[0]=olddimX;
  
  if ((error=gmcal_basic_matrix_new(newX, newY, newZ, newdimX[0], newdimY[0]))) {
    return error;
  }

  for (i=0; i<radius; i++) {
    for (j=0; j<newdimX[0]; j++) {
      newZ[0][radius-i-1][j]=newZ[0][radius+i][j]=(oldZ[middle-i-1][j]+oldZ[middle+i][j])/2.;
    }
    newY[0][radius-i-1]=oldY[middle-i-1];
    newY[0][radius+i]=oldY[middle+i];
  }

  for (i=0; i<newdimX[0]; i++) {
    newX[0][i]=oldX[i];
  }

  printf("dimX: %d, dimY: %d\n", newdimX[0], newdimY[0]);

  return GMCAL_SUCCESS;
}

static gint
gmcal_create_abel_matrix(gdouble ***matrix, gint n, gdouble *radii)
{
  gint i, j;
  gdouble ri, ri1;
  gdouble radius=radii[n];
  
  *matrix=(gdouble **) g_malloc((n+1)*sizeof(gdouble *));
  (*matrix)[n]=(gdouble *) g_malloc((n+1)*sizeof(gdouble));
  for (i=0; i<n; i++) {
    (*matrix)[i]=(gdouble *) g_malloc((n+1)*sizeof(gdouble));
    ri=radii[i];
    ri1=radii[i+1];
    (*matrix)[i][i]=ri1*ri1*PI/2.
    		   -ri*sqrt(ri1*ri1-ri*ri)
    		   -ri1*ri1*asin(ri/ri1);
    for (j=i+1; j<n; j++) {
      (*matrix)[i][j]=ri1*sqrt(radii[j+1]*radii[j+1]-ri1*ri1)
      		     +radii[j+1]*radii[j+1]*asin(ri1/radii[j+1])
      		     -ri1*sqrt(radii[j]*radii[j]-ri1*ri1)
      		     -radii[j]*radii[j]*asin(ri1/radii[j])
      		     -ri*sqrt(radii[j+1]*radii[j+1]-ri*ri)
      		     -radii[j+1]*radii[j+1]*asin(ri/radii[j+1])
      		     +ri*sqrt(radii[j]*radii[j]-ri*ri)
      		     +radii[j]*radii[j]*asin(ri/radii[j]);
    }
    (*matrix)[n][i]=PI/2.*(ri1*ri1-ri*ri);
    for (j=0; j<n; j++) {
      (*matrix)[i][j]*=n/radius;
      (*matrix)[i][n]+=(*matrix)[i][j];
    }
  }

  return GMCAL_SUCCESS;
}

static void
gmcal_abel_transform_vector(gdouble **matrix, gdouble *f, gdouble *h, gint n)
{
  int i; 
  int j;
  
  f=f+n;
  h=h+n;
  
  for (i=0; i<n; i++) {
    h[i]=0;
    for (j=0; j<n; j++) {
      h[i]+=matrix[i][j]*f[j];
    }
    h[-1-i]=h[i];
  }
}

gint	gmcal_abel_create_transformed_copy	(gfloat *oldX,
						gfloat *oldY,
						gfloat **oldZ,
						gint olddimX,
						gint olddimY,
						gfloat **newX,
						gfloat **newY,
						gfloat ***newZ,
						gint *newdimX,
						gint *newdimY,
						gint not_used_yet)
{
  gint error;
  gint i;
  gint j;
  gint len;
  gdouble *radii;
  gdouble **abelMatrix;
  gdouble radius;
  gdouble *f;
  gdouble *h;

  radius=1;  

  if ((error=gmcal_basic_matrix_check_integrity(oldX, oldY, oldZ, olddimX, olddimY))) {
    return error;
  }
  
  newdimY[0]=olddimY;
  newdimX[0]=olddimX;

  len=olddimY/2;
  
  if (len<4) {
    return GMCAL_ABEL_MATRIX_TOO_SMALL;
  }
  
  radii=(gdouble *) g_malloc((len+1)*sizeof(gdouble));
  for (i=0; i<len+1; i++) {
    radii[i]=radius/len*i;
  }

  if ((error=gmcal_create_abel_matrix(&abelMatrix, len, radii))) {
    return error;
  }

  if ((error=gmcal_basic_matrix_new(newX, newY, newZ, newdimX[0], newdimY[0]))) {
    return error;
  }

  f=(gdouble *) g_malloc(olddimY*sizeof(gdouble));
  h=(gdouble *) g_malloc(olddimY*sizeof(gdouble));
  
  for (i=0; i<olddimX; i++) {
    for (j=0; j<olddimY; j++) {
      f[j]=oldZ[j][i];
    }
    gmcal_abel_transform_vector(abelMatrix, f, h, len);
    for (j=0; j<olddimY; j++) {
      newZ[0][j][i]=h[j];
    }
  }  

  for (i=0; i<olddimX; i++) {
    newX[0][i]=oldX[i];
  }
  for (i=0; i<olddimY; i++) {
    newY[0][i]=oldY[i];
  }

  return GMCAL_SUCCESS;
}

gint	gmcal_abel_create_inverse_by_matrix	(gfloat *oldX,
						gfloat *oldY,
						gfloat **oldZ,
						gint olddimX,
						gint olddimY,
						gfloat **newX,
						gfloat **newY,
						gfloat ***newZ,
						gint *newdimX,
						gint *newdimY,
						gint not_used_yet)
{
  gint error;
  gint i;
  gint j;
  gint len;
  gdouble *radii;
  gdouble **abelMatrix;
  gdouble radius;
  gdouble *f;
  gdouble *h;

  radius=1;  

  if ((error=gmcal_basic_matrix_check_integrity(oldX, oldY, oldZ, olddimX, olddimY))) {
    return error;
  }
  
  newdimY[0]=olddimY;
  newdimX[0]=olddimX;

  len=olddimY/2;
  
  if (len<4) {
    return GMCAL_ABEL_MATRIX_TOO_SMALL;
  }
  
  radii=(gdouble *) g_malloc((len+1)*sizeof(gdouble));
  for (i=0; i<len+1; i++) {
    radii[i]=radius/len*i;
  }

  if ((error=gmcal_create_abel_matrix(&abelMatrix, len, radii))) {
    return error;
  }

  if ((error=gmcal_basic_matrix_double_invert(abelMatrix, len))) {
    return error;
  }

  if ((error=gmcal_basic_matrix_new(newX, newY, newZ, newdimX[0], newdimY[0]))) {
    return error;
  }

  f=(gdouble *) g_malloc(olddimY*sizeof(gdouble));
  h=(gdouble *) g_malloc(olddimY*sizeof(gdouble));
  
  for (i=0; i<olddimX; i++) {
    for (j=0; j<olddimY; j++) {
      f[j]=oldZ[j][i];
    }
    gmcal_abel_transform_vector(abelMatrix, f, h, len);
    for (j=0; j<olddimY; j++) {
      newZ[0][j][i]=h[j];
    }
  }  

  for (i=0; i<olddimX; i++) {
    newX[0][i]=oldX[i];
  }
  for (i=0; i<olddimY; i++) {
    newY[0][i]=oldY[i];
  }

  return GMCAL_SUCCESS;
}

static void gmcal_abel_back_distribute_vector(gdouble **matrix, gdouble *f, gdouble *h, gint n, gdouble *radii)
{
  gint i;
  gint j;
  
  f=f+n;
  h=h+n;

  for (i=0; i<n; i++) f[i]=0;
  for (i=0; i<n; i++) {
    for (j=0; j<n; j++) {
      f[j]+=(h[i]/matrix[i][n])*matrix[i][j]/matrix[n][j];
    }
  }
  for (i=0; i<n; i++) {
    f[i]*=radii[n]/n;
    f[-1-i]=f[i];
  }
}

gint	gmcal_abel_create_inverse_by_back_distribution(gfloat *oldX,
						gfloat *oldY,
						gfloat **oldZ,
						gint olddimX,
						gint olddimY,
						gfloat **newX,
						gfloat **newY,
						gfloat ***newZ,
						gint *newdimX,
						gint *newdimY,
						gint iterations)
{
  gint error;
  gint i;
  gint j;
  gint k;
  gint len;
  gdouble *radii;
  gdouble **abelMatrix;
  gdouble radius;
  gdouble *f;
  gdouble *h;
  gdouble *nf;
  gdouble *nh;

  radius=1;  

  if ((error=gmcal_basic_matrix_check_integrity(oldX, oldY, oldZ, olddimX, olddimY))) {
    return error;
  }
  
  newdimY[0]=olddimY;
  newdimX[0]=olddimX;

  len=olddimY/2;
  
  if (len<4) {
    return GMCAL_ABEL_MATRIX_TOO_SMALL;
  }
  
  radii=(gdouble *) g_malloc((len+1)*sizeof(gdouble));
  for (i=0; i<len+1; i++) {
    radii[i]=radius/len*i;
  }

  if ((error=gmcal_create_abel_matrix(&abelMatrix, len, radii))) {
    return error;
  }

  if ((error=gmcal_basic_matrix_double_invert(abelMatrix, len))) {
    return error;
  }

  if ((error=gmcal_basic_matrix_new(newX, newY, newZ, newdimX[0], newdimY[0]))) {
    return error;
  }

  f=(gdouble *) g_malloc(olddimY*sizeof(gdouble));
  h=(gdouble *) g_malloc(olddimY*sizeof(gdouble));
  nf=(gdouble *) g_malloc(olddimY*sizeof(gdouble));
  nh=(gdouble *) g_malloc(olddimY*sizeof(gdouble));
  
  for (i=0; i<olddimX; i++) {
    for (j=0; j<olddimY; j++) {
      h[j]=oldZ[j][i];
    }

    gmcal_abel_back_distribute_vector(abelMatrix, nf, h, len, radii);
    for (j=0; j<len*2; j++) {
       f[j]=nf[j];
    }
    gmcal_abel_transform_vector(abelMatrix, f, nh, len);
    for (j=0; j<len; j++) {
      /*    nh[len-j-1]=nh[len+j]=h[len+j]-nh[len+j];
      */
      nh[len-j-1]=nh[len+j]=(h[len+j]-nh[len+j])*(1+sqrt(radius*radius-radii[j]*radii[j]));
    }

    for (j=0; j<iterations; j++) {
      gmcal_abel_back_distribute_vector(abelMatrix, nf, nh, len, radii);
      for (k=0; k<len*2; k++) {
        f[k]+=nf[k];
      }
      gmcal_abel_transform_vector(abelMatrix, f, nh, len);
      for (k=0; k<len; k++) {
/*        nh[len-k-1]=nh[len+k]=h[len+k]-nh[len+k];
*/
        nh[len-k-1]=nh[len+k]=
        (h[len+k]-nh[len+k])*(1+sqrt(radius*radius-radii[k]*radii[k]));

      }
/*    tester=0;
      for (k=0; k<len*2; k++) {
        printf("f[%d]=%g\n",k,f[k]);
        tester+=nf[k];
      }
      printf("tester: %g\n",tester);
*/
    }

    for (j=0; j<olddimY; j++) {
      newZ[0][j][i]=f[j];
    }
  }  

  for (i=0; i<olddimX; i++) {
    newX[0][i]=oldX[i];
  }
  for (i=0; i<olddimY; i++) {
    newY[0][i]=oldY[i];
  }

  return GMCAL_SUCCESS;
}

gint	gmcal_abel_create_inverse_by_spline_expansion_real(gfloat *oldX,
						gfloat *oldY,
						gfloat **oldZ,
						gint olddimX,
						gint olddimY,
						gfloat **newX,
						gfloat **newY,
						gfloat ***newZ,
						gint *newdimX,
						gint *newdimY,
						gint knotN)
{
  gint error;
  gint i;
  gint j;
  gint k;
  gint l;
  gint len;
  gint *hPos;
  gdouble *radii;
  gfloat *knotR;
  gfloat **knotF;
  gdouble **abelMatrix;
  gdouble **splineH;
  gdouble **matrix;
  gdouble **coeff;
  gdouble *inhomogen;
  gdouble *solution;
  gdouble radius;
  gdouble *f, *h;
  gfloat *r;
  register gdouble dummy;

  radius=1;  

  if ((error=gmcal_basic_matrix_check_integrity(oldX, oldY, oldZ, olddimX, olddimY))) {
    return error;
  }
  
  newdimX[0]=olddimX;
  newdimY[0]=olddimY;
  
  len=olddimY/2;
  
  if (len<4) {
    return GMCAL_ABEL_MATRIX_TOO_SMALL;
  }
  
  radii=(gdouble *) g_malloc((len+1)*sizeof(gdouble));
  for (i=0; i<len+1; i++) {
    radii[i]=radius/len*i;
  }

  hPos=(gint *) g_malloc0(olddimY*sizeof(gint));
  h=(gdouble *) g_malloc0(olddimY*sizeof(gdouble));
  f=(gdouble *) g_malloc0(olddimY*sizeof(gdouble));
  r=(gfloat *) g_malloc0(olddimY*sizeof(gfloat));
  
/* Initialization of spline knots and matrix system for least squares fit */

  knotR=(gfloat *) g_malloc0((2*knotN-1)*sizeof(float));
  for (i=0; i<2*knotN-1; i++) {
    knotR[i]=0+(radius)/(knotN-1)*(i-knotN+1);
  }
  knotF=(gfloat **) g_malloc0(knotN*sizeof(gfloat *));
  coeff=(gdouble **) g_malloc0(knotN*sizeof(gdouble *));

  splineH=(gdouble **) g_malloc0(knotN*sizeof(gdouble *));

  matrix=(gdouble **) g_malloc0(knotN*sizeof(gdouble *));
  inhomogen=(gdouble *) g_malloc0(knotN*sizeof(gdouble));
  solution=(gdouble *) g_malloc0(knotN*sizeof(gdouble));

  for (i=0; i<knotN; i++) { 
    knotF[i]=(gfloat *) g_malloc0((2*knotN-1)*sizeof(gfloat));

    splineH[i]=(gdouble *) g_malloc0(olddimY*sizeof(gdouble));
    matrix[i]=(gdouble *) g_malloc0(knotN*sizeof(gdouble));


    knotF[i][knotN-1+i]=1;
    knotF[i][knotN-1-i]=1;
    
    gmcal_spline_get_coeff(knotR, knotF[i], 2*knotN-1, &coeff[i]);
  }

/* Look up spline positions for h(y) */
  for (i=0; i<len; i++) {
    r[len+i]=r[len-i-1]=i*radius/(len-1);
  }

  for (i=0; i<olddimY; i++) {
    hPos[i]=gmcal_spline_get_interval(knotR, 2*knotN-1, r[i]);
  }

  if ((error=gmcal_create_abel_matrix(&abelMatrix, len, radii))) {
    return error;
  }

  if ((error=gmcal_basic_matrix_new(newX, newY, newZ, newdimX[0], newdimY[0]))) {
    return error;
  }
  
/* Calculate abel transform of spline bases */
  for (i=0; i<knotN; i++) {
    for (j=0; j<olddimY; j++) {
      f[j]=gmcal_spline_get_value(knotR,knotF[i],2*knotN-1,coeff[i],r[j],hPos[j]);
    }
    gmcal_abel_transform_vector(abelMatrix, f, h, len); 
    for (j=0; j<olddimY; j++) {
      splineH[i][j]=h[j];
    }
  }


  for (l=0; l<olddimX; l++) {
  

/* Create matrix for least squares fit */
    for (i=0; i<knotN; i++) {
      for (j=i; j<knotN; j++) {
        dummy=0;
        for (k=0; k<len; k++) {
          dummy+=splineH[i][k]*splineH[j][k];
        }
        matrix[i][j]=matrix[j][i]=dummy;
/*        printf("%5.5f\t",matrix[i][j]);
*/
      }
      dummy=0;
      for (k=0; k<len; k++) {
        dummy+=oldZ[k][l]*splineH[i][k];
      }
      inhomogen[i]=dummy;
/*printf("= %5.5f bzw. \n",inhomogen[i]);
*/
    }
  

    gmcal_basic_matrix_double_solve_equ_sys(matrix,knotN,inhomogen,solution);

    for (i=0; i<olddimY; i++) {
      newZ[0][i][l]=0;
      for (j=0; j<knotN; j++) {
        newZ[0][i][l]+=solution[j]*gmcal_spline_get_value(knotR,knotF[j],2*knotN-1,coeff[j],r[i],hPos[i]);
      }
    }
  }

  for (i=0; i<olddimX; i++) {
    newX[0][i]=oldX[i];
  }
  for (i=0; i<olddimY; i++) {
    newY[0][i]=oldY[i];
  }

  return GMCAL_SUCCESS;
}

gint	gmcal_abel_create_inverse_by_fourier_expansion_real(gfloat *oldX,
						gfloat *oldY,
						gfloat **oldZ,
						gint olddimX,
						gint olddimY,
						gfloat **newX,
						gfloat **newY,
						gfloat ***newZ,
						gint *newdimX,
						gint *newdimY,
						gint knotN)
{
  gint error;
  gint i;
  gint j;
  gint k;
  gint l;
  gint len;
  gdouble *radii;
  gdouble **abelMatrix;
  gdouble **fourierH;
  gdouble **matrix;
  gdouble *inhomogen;
  gdouble *solution;
  gdouble radius;
  gdouble *f, *h;
  gdouble *r;
  register gdouble dummy;

  radius=1;  

  if ((error=gmcal_basic_matrix_check_integrity(oldX, oldY, oldZ, olddimX, olddimY))) {
    return error;
  }
  
  newdimX[0]=olddimX;
  newdimY[0]=olddimY;
  
  len=olddimY/2;
  
  if (len<4) {
    return GMCAL_ABEL_MATRIX_TOO_SMALL;
  }
  
  radii=(gdouble *) g_malloc((len+1)*sizeof(gdouble));
  for (i=0; i<len+1; i++) {
    radii[i]=radius/len*i;
  }

  h=(gdouble *) g_malloc0(olddimY*sizeof(gdouble));
  f=(gdouble *) g_malloc0(olddimY*sizeof(gdouble));
  r=(gdouble *) g_malloc0(olddimY*sizeof(gdouble));
  
  matrix=(gdouble **) g_malloc0(knotN*sizeof(gdouble *));
  inhomogen=(gdouble *) g_malloc0(knotN*sizeof(gdouble));
  solution=(gdouble *) g_malloc0(knotN*sizeof(gdouble));

  fourierH=(gdouble **) g_malloc0(knotN*sizeof(gdouble *));

  for (i=0; i<knotN; i++) { 
    fourierH[i]=(gdouble *) g_malloc0(olddimY*sizeof(gdouble));
    matrix[i]=(gdouble *) g_malloc0(knotN*sizeof(gdouble));
  }

  for (i=0; i<len; i++) {
    r[len+i]=r[len-i-1]=i*radius/(len-1);
  }

  if ((error=gmcal_create_abel_matrix(&abelMatrix, len, radii))) {
    return error;
  }

  if ((error=gmcal_basic_matrix_new(newX, newY, newZ, newdimX[0], newdimY[0]))) {
    return error;
  }
  
  /* Calculate abel transform of spline bases */
  for (i=0; i<knotN; i++) {
    for (j=0; j<olddimY; j++) {
      f[j]=1-(1-2*(i%2))*cos(i*PI*r[j]/radius);;
    }
    gmcal_abel_transform_vector(abelMatrix, f, h, len); 
    for (j=0; j<olddimY; j++) {
      fourierH[i][j]=h[j];
    }
  }


  for (l=0; l<olddimX; l++) {

    /* Create matrix for least squares fit */
    for (i=0; i<knotN; i++) {
      for (j=i; j<knotN; j++) {
        dummy=0;
        for (k=0; k<len; k++) {
          dummy+=fourierH[i][k]*fourierH[j][k];
        }
        matrix[i][j]=matrix[j][i]=dummy;
/*        printf("%5.5f\t",matrix[i][j]);
*/
      }
      dummy=0;
      for (k=0; k<len; k++) {
        dummy+=oldZ[k][l]*fourierH[i][k];
      }
      inhomogen[i]=dummy;
/*printf("= %5.5f bzw. \n",inhomogen[i]);
*/
    }
  

    gmcal_basic_matrix_double_solve_equ_sys(matrix,knotN,inhomogen,solution);

    for (i=0; i<olddimY; i++) {
      newZ[0][i][l]=0;
      for (j=0; j<knotN; j++) {
        newZ[0][i][l]+=1-(1-2*(i%2))*cos(i*PI*r[j]/radius);;
      }
    }
  }

  for (i=0; i<olddimX; i++) {
    newX[0][i]=oldX[i];
  }
  for (i=0; i<olddimY; i++) {
    newY[0][i]=oldY[i];
  }

  return GMCAL_SUCCESS;
}

gint	gmcal_abel_create_inverse_by_back_distribution_average(gfloat *oldX,
						gfloat *oldY,
						gfloat **oldZ,
						gint olddimX,
						gint olddimY,
						gfloat **newX,
						gfloat **newY,
						gfloat ***newZ,
						gint *newdimX,
						gint *newdimY,
						gint iterations)
{
  gint error;
  gint i;
  gint j;
  gint k;
  gint len;
  gdouble *radii;
  gdouble **abelMatrix;
  gdouble radius;
  gdouble *f;
  gdouble *h;
  gdouble *nf;
  gdouble *nh;
  gint average=(iterations+4)/5;
  gdouble **av_f;
  gint av_count;

  radius=1;  

  if ((error=gmcal_basic_matrix_check_integrity(oldX, oldY, oldZ, olddimX, olddimY))) {
    return error;
  }
  
  newdimY[0]=olddimY;
  newdimX[0]=olddimX;

  len=olddimY/2;
  
  if (len<4) {
    return GMCAL_ABEL_MATRIX_TOO_SMALL;
  }
  
  radii=(gdouble *) g_malloc((len+1)*sizeof(gdouble));
  for (i=0; i<len+1; i++) {
    radii[i]=radius/len*i;
  }

  if ((error=gmcal_create_abel_matrix(&abelMatrix, len, radii))) {
    return error;
  }

  if ((error=gmcal_basic_matrix_double_invert(abelMatrix, len))) {
    return error;
  }

  if ((error=gmcal_basic_matrix_new(newX, newY, newZ, newdimX[0], newdimY[0]))) {
    return error;
  }

  av_f=(gdouble **) g_malloc(average*sizeof(gdouble *));
  for (i=0; i<average; i++) {
    av_f[i]=(gdouble *) g_malloc(olddimY*sizeof(gdouble));
  }
  h=(gdouble *) g_malloc(olddimY*sizeof(gdouble));
  nf=(gdouble *) g_malloc(olddimY*sizeof(gdouble));
  nh=(gdouble *) g_malloc(olddimY*sizeof(gdouble));
  
  
  for (i=0; i<olddimX; i++) {
    av_count=0;
    f=av_f[av_count];
    for (j=0; j<olddimY; j++) {
      h[j]=oldZ[j][i];
    }

    gmcal_abel_back_distribute_vector(abelMatrix, nf, h, len, radii);
    for (j=0; j<len*2; j++) {
       f[j]=nf[j];
    }
    gmcal_abel_transform_vector(abelMatrix, f, nh, len);
    for (j=0; j<len; j++) {
      /*    nh[len-j-1]=nh[len+j]=h[len+j]-nh[len+j];
      */
      nh[len-j-1]=nh[len+j]=(h[len+j]-nh[len+j])*(1+sqrt(radius*radius-radii[j]*radii[j]));
    }

    for (j=0; j<iterations; j++) {
      gmcal_abel_back_distribute_vector(abelMatrix, nf, nh, len, radii);
      av_count=(av_count+1)%average;
      for (k=0; k<len*2; k++) {
        av_f[av_count][k]=f[k]+nf[k];
      }
      f=av_f[av_count];
      gmcal_abel_transform_vector(abelMatrix, f, nh, len);
      for (k=0; k<len; k++) {
/*        nh[len-k-1]=nh[len+k]=h[len+k]-nh[len+k];
*/
        nh[len-k-1]=nh[len+k]=
        (h[len+k]-nh[len+k])*(1+sqrt(radius*radius-radii[k]*radii[k]));

      }
/*    tester=0;
      for (k=0; k<len*2; k++) {
        printf("f[%d]=%g\n",k,f[k]);
        tester+=nf[k];
      }
      printf("tester: %g\n",tester);
*/
    }

    for (j=0; j<olddimY; j++) {
      newZ[0][j][i]=0;
      for (k=0; k<average; k++) {
        newZ[0][j][i]+=av_f[k][j];
      }
    }
  }  

  for (i=0; i<olddimX; i++) {
    newX[0][i]=oldX[i];
  }
  for (i=0; i<olddimY; i++) {
    newY[0][i]=oldY[i];
  }

  return GMCAL_SUCCESS;
}

gint	gmcal_abel_create_inverse_by_pre_calculated_back_distribution_average(gfloat *oldX,
						gfloat *oldY,
						gfloat **oldZ,
						gint olddimX,
						gint olddimY,
						gfloat **newX,
						gfloat **newY,
						gfloat ***newZ,
						gint *newdimX,
						gint *newdimY,
						gint iterations)
{
  gint error;
  gint i;
  gint j;
  gint k;
  gint len;
  gdouble *radii;
  gdouble **abelMatrix;
  gdouble radius;
  gdouble *f;
  gdouble *h;
  gdouble *nf;
  gdouble *nh;
  gint average=(iterations+4)/5;
  gdouble **av_f;
  gint av_count;

  radius=1;  

  if ((error=gmcal_abel_create_inverse_by_spline_expansion_real(oldX, oldY, oldZ, olddimX, olddimY,
								newX, newY, newZ, newdimX, newdimY,
								MAX(olddimY/5,3)))) {
    return error;
  }
  
  len=olddimY/2;
  
  if (len<4) {
    return GMCAL_ABEL_MATRIX_TOO_SMALL;
  }
  
  radii=(gdouble *) g_malloc((len+1)*sizeof(gdouble));
  for (i=0; i<len+1; i++) {
    radii[i]=radius/len*i;
  }

  if ((error=gmcal_create_abel_matrix(&abelMatrix, len, radii))) {
    return error;
  }

  if ((error=gmcal_basic_matrix_double_invert(abelMatrix, len))) {
    return error;
  }

  av_f=(gdouble **) g_malloc(average*sizeof(gdouble *));
  for (i=0; i<average; i++) {
    av_f[i]=(gdouble *) g_malloc(olddimY*sizeof(gdouble));
  }
  h=(gdouble *) g_malloc(olddimY*sizeof(gdouble));
  nf=(gdouble *) g_malloc(olddimY*sizeof(gdouble));
  nh=(gdouble *) g_malloc(olddimY*sizeof(gdouble));
  
  for (i=0; i<olddimX; i++) {
    av_count=0;
    f=av_f[av_count];
    for (j=0; j<olddimY; j++) {
      h[j]=oldZ[j][i];
    }

    for (j=0; j<len*2; j++) {
       f[j]=newZ[0][i][j];
    }
    gmcal_abel_transform_vector(abelMatrix, f, nh, len);
    for (j=0; j<len; j++) {
      /*    nh[len-j-1]=nh[len+j]=h[len+j]-nh[len+j];
      */
      nh[len-j-1]=nh[len+j]=(h[len+j]-nh[len+j])*(1+sqrt(radius*radius-radii[j]*radii[j]));
    }

    for (j=0; j<iterations; j++) {
      gmcal_abel_back_distribute_vector(abelMatrix, nf, nh, len, radii);
      av_count=(av_count+1)%average;
      for (k=0; k<len*2; k++) {
        av_f[av_count][k]=f[k]+nf[k];
      }
      f=av_f[av_count];
      gmcal_abel_transform_vector(abelMatrix, f, nh, len);
      for (k=0; k<len; k++) {
/*        nh[len-k-1]=nh[len+k]=h[len+k]-nh[len+k];
*/
        nh[len-k-1]=nh[len+k]=
        (h[len+k]-nh[len+k])*(1+sqrt(radius*radius-radii[k]*radii[k]));

      }
/*    tester=0;
      for (k=0; k<len*2; k++) {
        printf("f[%d]=%g\n",k,f[k]);
        tester+=nf[k];
      }
      printf("tester: %g\n",tester);
*/
    }

    for (j=0; j<olddimY; j++) {
      newZ[0][j][i]=0;
      for (k=0; k<average; k++) {
        newZ[0][j][i]+=av_f[k][j];
      }
    }
  }  

  for (i=0; i<olddimX; i++) {
    newX[0][i]=oldX[i];
  }
  for (i=0; i<olddimY; i++) {
    newY[0][i]=oldY[i];
  }

  return GMCAL_SUCCESS;
}

