/* gmcalbasic.c */

#include <string.h>
#include <gmcal/gmcalbasic.h>

gint gmcal_basic_matrix_new		(gfloat **X,
					gfloat **Y,
					gfloat ***Z,
					gint dimX,
					gint dimY)
{
  gint i;
  
  if (!X) return GMCAL_X_IS_NULL;
  if (X[0]) return GMCAL_X_0_NOT_NULL;
  if (!Y) return GMCAL_Y_IS_NULL;
  if (Y[0]) return GMCAL_Y_0_NOT_NULL;
  if (!Z) return GMCAL_Z_IS_NULL;
  if (Z[0]) return GMCAL_Z_0_NOT_NULL;
  if (dimX<1 || dimY<1) return GMCAL_DIMENSION_LESS_THAN_1;
  
  X[0]=(gfloat *) g_malloc0(dimX*sizeof(gfloat));
  if (!X[0]) return GMCAL_NOT_ENOUGH_MEMORY;
  Y[0]=(gfloat *) g_malloc0(dimY*sizeof(gfloat));
  if (!Y[0]) {
    g_free((void *) X[0]);
    X[0]=NULL;
    return GMCAL_NOT_ENOUGH_MEMORY;
  }
  Z[0]=(gfloat **) g_malloc0(dimY*sizeof(gfloat *));
  if (!Z[0]) {
    g_free((void *) X[0]);
    X[0]=NULL;
    g_free((void *) Y[0]);
    Y[0]=NULL;
    return GMCAL_NOT_ENOUGH_MEMORY;
  }
  for (i=0; i<dimY; i++) {
    Z[0][i]=(gfloat *) g_malloc0(dimX*sizeof(gfloat));
    if (!Z[0][i]) {
      gint j;
      
      g_free((void *) X[0]);
      X[0]=NULL;
      g_free((void *) Y[0]);
      Y[0]=NULL;
      for (j=0; j<i; j++) {
        g_free((void *) Z[0][j]);
      }
      g_free((void *) Z[0]);
      Z[0]=NULL;
      return GMCAL_NOT_ENOUGH_MEMORY;
    }
  }
  
  return GMCAL_SUCCESS;
}

gint gmcal_basic_matrix_free		(gfloat **X,
					gfloat **Y,
					gfloat ***Z,
					gint dimX,
					gint dimY)
{
  gint i;
  
  if (!X) return GMCAL_X_IS_NULL;
  if (!X[0]) return GMCAL_X_0_IS_NULL;
  if (!Y) return GMCAL_Y_IS_NULL;
  if (!Y[0]) return GMCAL_Y_0_IS_NULL;
  if (!Z) return GMCAL_Z_IS_NULL;
  if (!Z[0]) return GMCAL_Z_0_IS_NULL;
  if (dimX<1 || dimY<1) return GMCAL_DIMENSION_LESS_THAN_1;
  
  g_free((void *) X[0]);
  X[0]=NULL;
  g_free((void *) Y[0]);
  Y[0]=NULL;
  for (i=0; i<dimY; i++) {
    g_free((void *) Z[0][i]);
  }
  g_free((void *) Z[0]);
  Z[0]=NULL;
  
  return GMCAL_SUCCESS;
}

gint gmcal_basic_vector_fill_with_index	(gfloat *X,
					gint dimX)
{
  gint i;
  
  if (!X) return GMCAL_X_IS_NULL;
  
  for (i=0; i<dimX; i++) {
    X[i]=(gfloat) i;
  }

  return GMCAL_SUCCESS;
}

gint	gmcal_basic_vector_check_integrity	(gfloat *X,
						gint dimX)
{
  if (dimX<1) return GMCAL_DIMENSION_LESS_THAN_1;
  if (!X) return GMCAL_X_IS_NULL;

  return GMCAL_SUCCESS;
}

gint	gmcal_basic_matrix_check_integrity	(gfloat *X,
						gfloat *Y,
						gfloat **Z,
						gint dimX,
						gint dimY)
{
  gint i;
  
  if (dimX<1 || dimY<1) return GMCAL_DIMENSION_LESS_THAN_1;
  if (!X) return GMCAL_X_IS_NULL;
  if (!Y) return GMCAL_Y_IS_NULL;
  if (!Z) return GMCAL_Z_IS_NULL;
  for (i=0; i<dimY; i++) {
    if (!Z[i]) return GMCAL_Z_0_IS_NULL;
  }
  
  return GMCAL_SUCCESS;
}

gint	gmcal_basic_matrix_core_check_integrity	(gfloat **Z,
						gint dimX,
						gint dimY)
{
  gint i;
  
  if (dimX<1 || dimY<1) return GMCAL_DIMENSION_LESS_THAN_1;
  if (!Z) return GMCAL_Z_IS_NULL;
  for (i=0; i<dimY; i++) {
    if (!Z[i]) return GMCAL_Z_0_IS_NULL;
  }
  
  return GMCAL_SUCCESS;
}

gint gmcal_basic_matrix_double_invert	(gdouble **matrix, 
					gint n)
{
#define SWAP(a,b) {temp=(a);(a)=(b);(b)=temp;}

  int *indxc,*indxr,*ipiv;
  int i,icol,irow,j,k,l,ll;
  double big,dum,pivinv,temp;
  
  icol=0;
  irow=0;

  indxc=(int *) g_malloc0(n*sizeof(int));
  indxr=(int *) g_malloc0(n*sizeof(int));
  ipiv=(int *) g_malloc0(n*sizeof(int));
  for (j=0; j<n;j++) ipiv[j]=0;
  for (i=0; i<n;i++) {
    big=0.0;
    for (j=0; j<n;j++)
      if (ipiv[j] != 1)
        for (k=0; k<n;k++) {
          if (ipiv[k] == 0) {
            if (ABS(matrix[j][k]) >= big) {
              big=ABS(matrix[j][k]);
              irow=j;
              icol=k;
            }
          } else if (ipiv[k] > 1) {
            return GMCAL_SINGULAR_MATRIX;
          }
        }
    ++(ipiv[icol]);
    if (irow != icol) {
      for (l=0; l<n;l++) SWAP(matrix[irow][l],matrix[icol][l])
    }
    indxr[i]=irow;
    indxc[i]=icol;
    if (matrix[icol][icol] == 0.0) {
      return GMCAL_SINGULAR_MATRIX;
    }
    pivinv=1.0/matrix[icol][icol];
    matrix[icol][icol]=1.0;
    for (l=0; l<n;l++) matrix[icol][l] *= pivinv;
    for (ll=0; ll<n;ll++)
      if (ll != icol) {
        dum=matrix[ll][icol];
        matrix[ll][icol]=0.0;
        for (l=0; l<n;l++) matrix[ll][l] -= matrix[icol][l]*dum;
      }
  }
  for (l=n-1;l>=0; l--) {
    if (indxr[l] != indxc[l])
       for (k=0; k<n;k++)
        SWAP(matrix[k][indxr[l]],matrix[k][indxc[l]]);
  }
  g_free((void *) ipiv);
  g_free((void *) indxr);
  g_free((void *) indxc);

#undef SWAP
  return GMCAL_SUCCESS;
}

gint 	gmcal_basic_matrix_double_solve_equ_sys	(gdouble **matrix, 
						gint n,
						gdouble *inhomogen,
						double *solution)
{
    gint i,j,k;         /*  Zaehlvariablen                                     */
    gdouble doub;       /*  Rechengroesse                                      */
    gdouble  *z;        /*  Vektor zum Zwischenspeichern                       */
    gdouble **a;        /*  Untere Dreiecksmatrix alpha (Num. Recipes)         */
    gdouble **b;        /*  Obere  Dreiecksmatrix beta  (Num. Recipes)         */

    z = (gdouble *) g_malloc0(n*sizeof(gdouble));
    a = (gdouble **) g_malloc0(n*sizeof(gdouble *));
    for (i = 0; i < n; i++)
	    a[i] = (gdouble *) g_malloc0(n*sizeof(gdouble));
    b = (double **) g_malloc0(n*sizeof(gdouble *));
    for (i = 0; i < n; i++)
	    b[i] = (gdouble *) g_malloc0(n*sizeof(gdouble));



    for (j = 0; j <= n-1; j++)
    {
	a[j][j] = 1.0;

	for (i = 0; i <= j; i++)
	{
	    doub = 0.0;
	    for (k = 0; k <= i-1; k++)
	    {
		doub += a[i][k] * b[k][j];
	    }
	    b[i][j] = matrix[i][j] - doub;
	}

	for (i = j+1; i <= n-1; i++)
	{
	    doub = 0.0;
	    for (k = 0; k <= j-1; k++)
	    {
		doub += a[i][k] * b[k][j];
	    }
	    a[i][j] = (matrix[i][j] - doub) / b[j][j];
	}
    }

    for (i = 0; i <= n-1; i++)
    {
	doub = 0.0;

	for (j = 0; j < i; j++)
	{
	    doub += a[i][j] * z[j];
	}

	z[i] = (inhomogen[i] - doub) / a[i][i];
    }

    for (i = n-1; i >= 0; i--)
    {
	doub = 0.0;

	for (j = n-1; j > i; j--)
	{
	    doub += b[i][j] * solution[j];
	}

	solution[i] = (z[i] - doub) / b[i][i];
    }

    g_free (z);
    for (i = 0; i < n; i++)   g_free (a[i]);
    g_free (a);
    for (i = 0; i < n; i++)   g_free (b[i]);
    g_free (b);

  return GMCAL_SUCCESS;
}