/* gmcalsimple.c */

#include <string.h>
#include <gmcal/gmcalbasic.h>
#include <gmcal/gmcalsimple.h>

gint gmcal_simple_matrix_multiply_number		(gfloat **Z,
							gint dimX,
							gint dimY,
							gfloat number)
{
  gint i, j;
  gint error;
  
  if ((error=gmcal_basic_matrix_core_check_integrity(Z, dimX, dimY))) {
    return error;
  }
  
  for (i=0; i<dimY; i++) {
    for (j=0; j<dimX; j++) {
      Z[i][j]*=number;
    }
  }
  
  return GMCAL_SUCCESS;
}

gint gmcal_simple_matrix_divide_number			(gfloat **Z,
							gint dimX,
							gint dimY,
							gfloat number)
{
  gint i, j;
  gint error;
  
  if ((error=gmcal_basic_matrix_core_check_integrity(Z, dimX, dimY))) {
    return error;
  }
  
  if (!number) {
    return GMCAL_DIVISION_BY_ZERO;
  }
  
  for (i=0; i<dimY; i++) {
    for (j=0; j<dimX; j++) {
      Z[i][j]/=number;
    }
  }
  
  return GMCAL_SUCCESS;
}

gint gmcal_simple_matrix_add_number			(gfloat **Z,
							gint dimX,
							gint dimY,
							gfloat number)
{
  gint i, j;
  gint error;
  
  if ((error=gmcal_basic_matrix_core_check_integrity(Z, dimX, dimY))) {
    return error;
  }
  
  if (!number) {
    return GMCAL_SUCCESS;
  }
  
  for (i=0; i<dimY; i++) {
    for (j=0; j<dimX; j++) {
      Z[i][j]+=number;
    }
  }
  
  return GMCAL_SUCCESS;
}

gint gmcal_simple_matrix_subtract_number		(gfloat **Z,
							gint dimX,
							gint dimY,
							gfloat number)
{
  gint i, j;
  gint error;
  
  if ((error=gmcal_basic_matrix_core_check_integrity(Z, dimX, dimY))) {
    return error;
  }
  
  if (!number) {
    return GMCAL_SUCCESS;
  }
  
  for (i=0; i<dimY; i++) {
    for (j=0; j<dimX; j++) {
      Z[i][j]-=number;
    }
  }
  
  return GMCAL_SUCCESS;
}

gint gmcal_simple_matrix_multiply_matrix_times_number	(gfloat **Z,
							gfloat **Z1,
							gint dimX,
							gint dimY,
							gfloat number)
{
  gint i, j;
  gint error;
  
  if ((error=gmcal_basic_matrix_core_check_integrity(Z, dimX, dimY))) {
    return error;
  }
  
  if ((error=gmcal_basic_matrix_core_check_integrity(Z1, dimX, dimY))) {
    return error;
  }
 
  for (i=0; i<dimY; i++) {
    for (j=0; j<dimX; j++) {
      Z[i][j]*=number*Z1[i][j];
    }
  }
  
  return GMCAL_SUCCESS;
}

gint gmcal_simple_matrix_divide_matrix_times_number	(gfloat **Z,
							gfloat **Z1,
							gint dimX,
							gint dimY,
							gfloat number)
{
  gint i, j;
  gint error;
  
  if ((error=gmcal_basic_matrix_core_check_integrity(Z, dimX, dimY))) {
    return error;
  }
  
  if ((error=gmcal_basic_matrix_core_check_integrity(Z1, dimX, dimY))) {
    return error;
  }
 
  if (!number) {
    return GMCAL_DIVISION_BY_ZERO;
  }
  
  for (i=0; i<dimY; i++) {
    for (j=0; j<dimX; j++) {
      Z[i][j]/=number*Z1[i][j];
    }
  }
  
  return GMCAL_SUCCESS;
}

gint gmcal_simple_matrix_add_matrix_times_number	(gfloat **Z,
							gfloat **Z1,
							gint dimX,
							gint dimY,
							gfloat number)
{
  gint i, j;
  gint error;
  
  if ((error=gmcal_basic_matrix_core_check_integrity(Z, dimX, dimY))) {
    return error;
  }
  
  if ((error=gmcal_basic_matrix_core_check_integrity(Z1, dimX, dimY))) {
    return error;
  }
 
  if (!number) {
    return GMCAL_SUCCESS;
  }
  
  for (i=0; i<dimY; i++) {
    for (j=0; j<dimX; j++) {
      Z[i][j]+=(number*Z1[i][j]);
    }
  }
  
  return GMCAL_SUCCESS;
}

gint gmcal_simple_matrix_subtract_matrix_times_number	(gfloat **Z,
							gfloat **Z1,
							gint dimX,
							gint dimY,
							gfloat number)
{
  gint i, j;
  gint error;
  
  if ((error=gmcal_basic_matrix_core_check_integrity(Z, dimX, dimY))) {
    return error;
  }
  
  if ((error=gmcal_basic_matrix_core_check_integrity(Z1, dimX, dimY))) {
    return error;
  }
 
  if (!number) {
    return GMCAL_SUCCESS;
  }
  
  for (i=0; i<dimY; i++) {
    for (j=0; j<dimX; j++) {
      Z[i][j]-=(number*Z1[i][j]);
    }
  }
  
  return GMCAL_SUCCESS;
}

gint gmcal_simple_create_matrix_transposed		(gfloat *oldX,
							gfloat *oldY,
							gfloat **oldZ,
							gint olddimX,
							gint olddimY,
							gfloat **newX,
							gfloat **newY,
							gfloat ***newZ,
							gint *newdimX,
							gint *newdimY)
{
  gint i, j;
  gint error;
  
  if ((error=gmcal_basic_matrix_check_integrity(oldX, oldY, oldZ, olddimX, olddimY))) {
    return error;
  }
  
  newdimY[0]=olddimX;
  newdimX[0]=olddimY;

  if ((error=gmcal_basic_matrix_new(newX, newY, newZ, newdimX[0], newdimY[0]))) {
    return error;
  }
 
  for (i=0; i<olddimX; i++) {
    newY[0][i]=oldX[i];
  }
  for (i=0; i<olddimY; i++) {
    newX[0][i]=oldY[i];
  }

  for (i=0; i<olddimY; i++) {
    for (j=0; j<olddimX; j++) {
      newZ[0][j][i]=oldZ[i][j];
    }
  }


 
  return GMCAL_SUCCESS;
}