#include <gmcal/gmcalbasic.h>
#include <gmcal/gmcalsimple.h>
#include <gmcal/gmcalspline.h>
#include <gmcal/gmcalread.h>
#include <gmcal/gmcalwrite.h>
#include <gmcal/gmcalabel.h>
#include <script_matrix.h>

void gmcal_script_init_matrix()
{
  first_data=NULL;
  last_data=NULL;
}

gmcal_data *gmcal_script_create_matrix(int dimX, int dimY)
{
  gmcal_data *data;
  int _error;
  
  data=(gmcal_data *) calloc(1,sizeof(gmcal_data));
  
  if (last_data) { 
    last_data->next=data;
    data->index=last_data->index+1;
  } else {
    data->index=1;
    first_data=data;
  }
  data->previous=last_data;
  last_data=data;
  
  data->dimX=dimX;
  data->dimY=dimY;
  data->info=NULL;

  if ((_error=gmcal_basic_matrix_new(&data->X, &data->Y, &data->Z, dimX, dimY))){
    warning("Error: %d\n", _error);
    return NULL;
  }

  return data;
}

gmcal_data *gmcal_script_create_matrix_hull()
{
  gmcal_data *data;
  
  data=(gmcal_data *) calloc(1,sizeof(gmcal_data));
  
  if (last_data) { 
    last_data->next=data;
    data->index=last_data->index+1;
  } else {
    data->index=1;
    first_data=data;
  }
  data->previous=last_data;
  last_data=data;
  
  data->dimX=0;
  data->dimY=0;
  data->X=NULL;
  data->Y=NULL;
  data->Z=NULL;
  data->info=NULL;

  return data;
}

int gmcal_script_delete_matrix(gmcal_data *data)
{
  int i;
  
  if (last_data==data) {
    last_data=data->previous;
  }
  
  if (first_data==data) {
    first_data=data->next;
  }
  
  if (data->next) {
    data->next->previous=data->previous;
  }
  
  if (data->previous) {
    data->previous->next=data->next;
  }
  
  if (data->info) {
    free((void *) data->info);
  }

  if (data->path) {
    free((void *) data->path);
  }
  
  free((void *) data->X);
  free((void *) data->Y);

  for (i=0; i<data->dimY; i++) {
    free((void *) data->Z[i]);
  }
  
  free((void *) data->Z);
  
  free((void *) data);
  
  return 1;
}

gmcal_data *gmcal_script_get_data_by_index(int index)
{
  gmcal_data *data;
  
  data=first_data;
  
  while(data) {
    if (data->index==index) return data;
    data=data->next;
  }
  
  return (gmcal_data *) NULL;
}

float gmcal_script_array_integral(float *array, float min, float max)
{
  float value;
  int imin;
  int imax;
  int i;
  
  imin=(int) min;
  imin++;
  imax=(int) max;
  
  
  if (imin>imax) {
    return array[imin-1]*(max-min);
  }
  
  value=array[imin-1]*(imin-min);
  for (i=imin; i<imax; i++) value+=array[i];
  value+=array[imax]*(max-imax);
  
  return value;
}

void gmcal_script_solve_quad_equ_sys_double(double **matrix, int n, double *inhomogen, double *solution)
/*	
	Input:
 	matrix		- matrix of equation system
	n		- height and width of matrix
	inhomogen	- inhomogenity vector
	
	Output:
	
	solution	- solution of the equation system
	
	
	Return:
	
	N.A.
*/
{
    int i,j,k;         /*  Zaehlvariablen                                     */
    double doub;       /*  Rechengroesse                                      */
    double  *z;        /*  Vektor zum Zwischenspeichern                       */
    double **a;        /*  Untere Dreiecksmatrix alpha (Num. Recipes)         */
    double **b;        /*  Obere  Dreiecksmatrix beta  (Num. Recipes)         */

    z = (double *) calloc(n,sizeof(double));
    a = (double **) calloc(n,sizeof(double *));
    for (i = 0; i < n; i++)
	    a[i] = (double *) calloc(n,sizeof(double));
    b = (double **) calloc(n,sizeof(double *));
    for (i = 0; i < n; i++)
	    b[i] = (double *) calloc(n,sizeof(double));



    for (j = 0; j <= n-1; j++)
    {
	a[j][j] = 1.0;

	for (i = 0; i <= j; i++)
	{
	    doub = 0.0;
	    for (k = 0; k <= i-1; k++)
	    {
		doub += a[i][k] * b[k][j];
	    }
	    b[i][j] = matrix[i][j] - doub;
	}

	for (i = j+1; i <= n-1; i++)
	{
	    doub = 0.0;
	    for (k = 0; k <= j-1; k++)
	    {
		doub += a[i][k] * b[k][j];
	    }
	    a[i][j] = (matrix[i][j] - doub) / b[j][j];
	}
    }

    for (i = 0; i <= n-1; i++)
    {
	doub = 0.0;

	for (j = 0; j < i; j++)
	{
	    doub += a[i][j] * z[j];
	}

	z[i] = (inhomogen[i] - doub) / a[i][i];
    }

    for (i = n-1; i >= 0; i--)
    {
	doub = 0.0;

	for (j = n-1; j > i; j--)
	{
	    doub += b[i][j] * solution[j];
	}

	solution[i] = (z[i] - doub) / b[i][i];
    }

    free (z);
    for (i = 0; i < n; i++)   free (a[i]);
    free (a);
    for (i = 0; i < n; i++)   free (b[i]);
    free (b);

    return;
}

/* -------------------------------------------------------------------
 *
 * -------------------------------------------------------------------*/  

int gmcal_script_new_matrix(int dimX, int dimY, float minX, float maxX, float minY, float maxY, float value)
{
  float factor;
  int i, j;
  gmcal_data *data;
  
  if (dimX<1) {
    dimX=1;
    warning("dimX less than 1! Using 1 instead\n");
  }

  if (dimY<1) {
    dimY=1;
    warning("dimY less than 1! Using 1 instead\n");
  }
  
  data=gmcal_script_create_matrix(dimX, dimY);
  
  if (value) {
    for (i=0; i<dimY; i++) {
      for (j=0; j<dimX; j++) {
        data->Z[i][j]=value;
      }
    }
  }
  
  if (dimX>1) {
    factor=(maxX-minX)/(dimX-1);
  } else {
    factor=0;
  }
  for (i=0; i<dimX; i++) {
     data->X[i]=minX+i*factor;
  }
  
  if (dimY>1) {
    factor=(maxY-minY)/(dimY-1);
  } else {
    factor=0;
  }
  for (i=0; i<dimY; i++) {
     data->Y[i]=minY+i*factor;
  }

  return data->index;
}

int gmcal_script_set_dispersion_x(int index, float min, float max)
{
  gmcal_data *data;
  int i;
  float factor;
  
  if (!(data=gmcal_script_get_data_by_index(index))) {
    error("No matrix with index %d is in memory\n", (int) index);
    return 0;
  }
  
  if (data->dimX>1) {
    factor=(max-min)/(data->dimX-1);
  } else {
    factor=0;
  }
  for (i=0; i<data->dimX; i++) {
     data->X[i]=min+i*factor;
  }
  
  return 1;  
  
}

int gmcal_script_destroy_matrix(int index)
{
  gmcal_data *data;
  
  if (!(data=gmcal_script_get_data_by_index(index))) {
    error("No matrix with index %d is in memory\n", (int) index);
    return 0;
  }
  
  gmcal_script_delete_matrix(data);
  
  return 1;
}

int gmcal_script_destroy_all_matrices()
{
  gmcal_data *data;
  gmcal_data *delete;
  
  data=first_data;
  
  while(data) {
    delete=data;
    data=data->next;
    gmcal_script_delete_matrix(delete);
  }

  return 1;
}

int gmcal_script_get_matrix_attributes(int index, int *dimX, int *dimY, float *minX, float *maxX, float *minY, float *maxY)
{
  gmcal_data *data;
  int i;
  
  if (!(data=gmcal_script_get_data_by_index(index))) {
    error("No matrix with index %d is in memory\n", (int) index);
    return 0;
  }
  
  *dimX=data->dimX;
  *dimY=data->dimY;
  
  *minX=*maxX=data->X[0];
  for (i=1; i<data->dimX; i++) {
    *minX=MIN(data->X[i], *minX);
    *maxX=MAX(data->X[i], *maxX);
  }
  
  *minY=*maxY=data->Y[0];
  for (i=1; i<data->dimY; i++) {
    *minY=MIN(data->Y[i], *minY);
    *maxY=MAX(data->Y[i], *maxY);
  }

  return 1;
}

int gmcal_script_set_matrix_entry(int index, int row, int col, float value)
{
  gmcal_data *data;
  int dimX;
  int dimY;
  
  if (!(data=gmcal_script_get_data_by_index(index))) {
    error("No matrix with index %d is in memory\n", (int) index);
    return 0;
  }

  dimX=data->dimX;
  dimY=data->dimY;
  
  if (col<0 || col>=dimX) {
    error("Col index (%d) must be between 0 and dimX-1 (%d); Nothing could be done\n", (int) col, dimX-1);
    return 0;
  }

  if (row<0 || row>=dimY) {
    error("Row index (%d) must be between 0 and dimY-1 (%d); Nothing could be done\n", (int) col, dimY-1);
    return 0;
  }
  
  data->Z[row][col]=value;

  return 1;
}

int gmcal_script_set_matrix_info(int index, char *info)
{
  gmcal_data *data;
  
  if (!(data=gmcal_script_get_data_by_index(index))) {
    error("No matrix with index %d is in memory\n", (int) index);
    return 0;
  }

  data->info=strdup(info);

  return 1;
}

int gmcal_script_copy_matrix(int index)
{
  gmcal_data *data;
  gmcal_data *copy;
  int dimX;
  int dimY;
  int i, j;
  
  if (!(data=gmcal_script_get_data_by_index(index))) {
    error("No matrix with index %d is in memory\n", (int) index);
    return 0;
  }

  dimX=data->dimX;
  dimY=data->dimY;
  
  copy=gmcal_script_create_matrix(dimX, dimY);
  
  for (i=0; i<dimX; i++) {
     copy->X[i]=data->X[i];
  }
  for (i=0; i<dimY; i++) {
     copy->Y[i]=data->Y[i];
  }
  for (i=0; i<dimY; i++) {
    for (j=0; j<dimX; j++) {
       copy->Z[i][j]=data->Z[i][j];
    }
  }
  
  return copy->index;
}

int gmcal_script_write_all_gif(char *path, char *model, int maxColors, int mode, int imagesPerRow)
{
  gmcal_data *data;
  int num;
  int i;
  float ***Z;
  float **X;
  float **Y;
  int *dimX;
  int *dimY;
  char **info;
  int _error;

  if (!(data=first_data)) {
    warning("No matrices in memory!\n");
    return 0;
  }
  
  num=1;
  while((data=data->next)) num++;
  
  printf("%d Matrices have been found...\n", num);
  
  Z=(float ***) malloc(num*sizeof(float **));
  X=(float **) malloc(num*sizeof(float *));
  Y=(float **) malloc(num*sizeof(float *));
  dimX=(int *) malloc(num*sizeof(int));
  dimY=(int *) malloc(num*sizeof(int));
  info=(char **) malloc(num*sizeof(char *));
  
  data=first_data;
  i=0;
  while(i<num) {
    Z[i]=data->Z;
    X[i]=data->X;
    Y[i]=data->Y;
    dimX[i]=data->dimX;
    dimY[i]=data->dimY;
    info[i]=data->info;
    i++;
    data=data->next;
  }
  
  if ((_error=gmcal_write_float_gif_matrices_with_info(path, X, Y, Z, dimX, dimY, info, num, model, maxColors, mode, imagesPerRow))) {
    warning("Error: %d\n", _error);
    return 0;
  }
  
  return 1;
}

int gmcal_script_write_gif(int index, char *path, char *model, int maxColors, int mode)
{
  gmcal_data *data;
  int _error;

  if (!(data=gmcal_script_get_data_by_index(index))) {
    error("No matrix with index %d is in memory\n", (int) index);
    return 0;
  }
  
  if ((_error=gmcal_write_float_gif_matrices_with_info(path, &data->X, &data->Y, &data->Z, &data->dimX, &data->dimY, &data->info, 1, model, maxColors, mode, 1))) {
    warning("Error: %d\n", _error);
    return 0;
  }

  return 1;
}

int gmcal_script_write_matrix(int index, char *path, int type)
{
  gmcal_data *data;
  int _error;
  int (*fnct)(char *path, float *X, float *Y, float **Z, int dimX, int dimY, char *info);

  if (!(data=gmcal_script_get_data_by_index(index))) {
    error("No matrix with index %d is in memory\n", (int) index);
    return 0;
  }
  
  switch (type) {
    case GMCAL_SCRIPT_ASCII:	fnct=gmcal_write_float_ascii_matrix_gmcal; break;
    case GMCAL_SCRIPT_BIN:	fnct=gmcal_write_float_bin_matrix_gmcal; break;
    default: 			warning("File type not implemented\n");
    				return 0;
  }
  
  if ((_error=fnct(path, data->X, data->Y, data->Z, data->dimX, data->dimY, data->info))) { 
    warning("Error: %d\n", _error);
    return 0;
  }

  return 1;
}

int gmcal_script_write_row_as_star(int index, int row, char *path)
{
  gmcal_data *data;
  int _error;

  if (!(data=gmcal_script_get_data_by_index(index))) {
    error("No matrix with index %d is in memory\n", (int) index);
    return 0;
  }
  

  if (row<0 || row>=data->dimY) {
    warning("Row %d not existent\n", row);
  }

  if ((_error=gmcal_write_float_bin_vector_simple(path, data->Z[row], data->dimX))) { 
    warning("Error: %d\n", _error);
    return 0;
  }

  return 1;
}

int gmcal_script_load_matrix(char *path, int type)
{
  gmcal_data *data;
  int i;
  int _error;
  float dummy;
  int (*fnct)(char *path, float **X, float **Y, float ***Z, int *dimX, int *dimY, char **info);

  data=gmcal_script_create_matrix_hull();
  
  switch (type) {
    case GMCAL_SCRIPT_ASCII:	fnct=gmcal_read_float_ascii_matrix_gmcal; break;
    case GMCAL_SCRIPT_BIN:	fnct=gmcal_read_float_bin_matrix_gmcal; break;
    case GMCAL_SCRIPT_BIN_VECTOR:fnct=gmcal_read_float_bin_vector_simple; break;
    case GMCAL_SCRIPT_BMP:	fnct=gmcal_read_float_bmp_matrix_pco; break;
    case GMCAL_SCRIPT_SPEC:	fnct=gmcal_read_float_bin_matrix_csma; break;
    case GMCAL_SCRIPT_OMA4:	fnct=gmcal_read_float_bin_matrix_oma4; break;
    default:			warning("File type not implemented.\n");
    				return 0;
  }
  
  if ((_error=fnct(path, &data->X, &data->Y, &data->Z, &data->dimX, &data->dimY, &data->info))) {
    warning("Error: %d\n", _error);
    return 0;
  }
  
  /* cut away first row (my framegrabber delivers the first line corrupted...) */

  if (type==GMCAL_SCRIPT_BMP && data->dimY>1) {
    free(data->Z[data->dimY-1]);
    data->dimY--;
  }
  
  if (type==GMCAL_SCRIPT_OMA4) {
    if (data->X[0]>data->X[data->dimX-1]) {
      warning("X-Dispersion has been changed...\n");
      for (i=0; i<data->dimX/2; i++) {
        dummy=data->X[i];
        data->X[i]=data->X[data->dimX-1-i];
        data->X[data->dimX-1-i]=dummy;
      }
    }
  }
  return data->index;
}

int gmcal_script_create_abel_copy(int index, int type, int iterations)
{
  gmcal_data *data;
  gmcal_data *copy;
  int _error;
  int (*fnct)(float *oldX, float *oldY, float **oldZ, int olddimX, int olddimY, 
  	float **newX, float **newY, float ***newZ, int *newdimX, int *newdimY, int iterations);
  
  if (!(data=gmcal_script_get_data_by_index(index))) {
    error("No matrix with index %d is in memory\n", (int) index);
    return 0;
  }

  copy=gmcal_script_create_matrix_hull();  
  
  switch (type) {
    case GMCAL_SCRIPT_ABEL_SYMMETRICAL:
    		fnct=gmcal_abel_create_symmetrical_copy; break;
    case GMCAL_SCRIPT_ABEL_TRANSFORM:
    		fnct=gmcal_abel_create_transformed_copy; break;
    case GMCAL_SCRIPT_ABEL_MATRIX_INVERSE:
    		fnct=gmcal_abel_create_inverse_by_matrix; break;
    case GMCAL_SCRIPT_ABEL_SPLINE_REAL_INVERSE:
    		fnct=gmcal_abel_create_inverse_by_spline_expansion_real; break;
    case GMCAL_SCRIPT_ABEL_FOURIER_REAL_INVERSE:
    		fnct=gmcal_abel_create_inverse_by_fourier_expansion_real; break;
    case GMCAL_SCRIPT_ABEL_BACK_DISTRIBUTION_INVERSE:
    		fnct=gmcal_abel_create_inverse_by_back_distribution; break;
    case GMCAL_SCRIPT_ABEL_AVERAGE_BACK_DISTRIBUTION_INVERSE:
    		fnct=gmcal_abel_create_inverse_by_back_distribution_average; break;
    case GMCAL_SCRIPT_ABEL_PRE_CALCULATED_AVERAGE_BACK_DISTRIBUTION_INVERSE:
    		fnct=gmcal_abel_create_inverse_by_pre_calculated_back_distribution_average; break;
    default: 	warning("Abel operation type not implemented.\n");
    		return 0;
  }
  
  if ((_error=fnct(data->X, data->Y, data->Z, data->dimX, data->dimY,
  		   &copy->X,&copy->Y,&copy->Z,&copy->dimX,&copy->dimY, iterations))) {
    warning("Error: %d\n", _error);
    return 0;
  }
  
  return copy->index;
}

int gmcal_script_create_transposed_matrix(int index)
{
  gmcal_data *data;
  gmcal_data *copy;
  int _error;
  
  if (!(data=gmcal_script_get_data_by_index(index))) {
    error("No matrix with index %d is in memory\n", (int) index);
    return 0;
  }

  copy=gmcal_script_create_matrix_hull();  
  
  if ((_error=gmcal_simple_create_matrix_transposed(data->X, data->Y, data->Z, data->dimX, data->dimY,
  						&copy->X,&copy->Y,&copy->Z,&copy->dimX,&copy->dimY))) {
    warning("Error: %d\n", _error);
    return 0;
  }

  return copy->index;
}

int gmcal_script_add_matrix_to_matrix(int index, int index2, float value)
{
  gmcal_data *data;
  gmcal_data *second;
  int _error;
  
  if (!(data=gmcal_script_get_data_by_index(index))) {
    error("No matrix with index %d is in memory\n", (int) index);
    return 0;
  }

  if (!(second=gmcal_script_get_data_by_index(index2))) {
    error("No matrix with index %d is in memory\n", (int) index2);
    return 0;
  }

  if ((_error=gmcal_simple_matrix_add_matrix_times_number(data->Z, second->Z, data->dimX, data->dimY, value))) {
    warning("Error: %d\n", _error);
    return 0;
  }

  return data->index;
}

int gmcal_script_multiply_matrix_by_matrix(int index, int index2, float value)
{
  gmcal_data *data;
  gmcal_data *second;
  int _error;
  
  if (!(data=gmcal_script_get_data_by_index(index))) {
    error("No matrix with index %d is in memory\n", (int) index);
    return 0;
  }

  if (!(second=gmcal_script_get_data_by_index(index2))) {
    error("No matrix with index %d is in memory\n", (int) index2);
    return 0;
  }

  if ((_error=gmcal_simple_matrix_multiply_matrix_times_number(data->Z, second->Z, data->dimX, data->dimY, value))) {
    warning("Error: %d\n", _error);
    return 0;
  }

  return data->index;
}

int gmcal_script_divide_matrix_by_matrix(int index, int index2, float value)
{
  gmcal_data *data;
  gmcal_data *second;
  int _error;
  
  if (!(data=gmcal_script_get_data_by_index(index))) {
    error("No matrix with index %d is in memory\n", (int) index);
    return 0;
  }

  if (!(second=gmcal_script_get_data_by_index(index2))) {
    error("No matrix with index %d is in memory\n", (int) index2);
    return 0;
  }
  
  if ((_error=gmcal_simple_matrix_divide_matrix_times_number(data->Z, second->Z, data->dimX, data->dimY, value))) {
    warning("Error: %d\n", _error);
    return 0;
  }

  return data->index;
}

int gmcal_script_add_number_to_matrix(int index, float value)
{
  gmcal_data *data;
  int _error;
  
  if (!(data=gmcal_script_get_data_by_index(index))) {
    error("No matrix with index %d is in memory\n", (int) index);
    return 0;
  }

  if ((_error=gmcal_simple_matrix_add_number(data->Z, data->dimX, data->dimY, value))) {
    warning("Error: %d\n", _error);
    return 0;
  }

  return data->index;
}

int gmcal_script_multiply_matrix_by_number(int index, float value)
{
  gmcal_data *data;
  int _error;
  
  if (!(data=gmcal_script_get_data_by_index(index))) {
    error("No matrix with index %d is in memory\n", (int) index);
    return 0;
  }

  if ((_error=gmcal_simple_matrix_multiply_number(data->Z, data->dimX, data->dimY, value))) {
    warning("Error: %d\n", _error);
    return 0;
  }

  return data->index;
}

int gmcal_script_divide_matrix_by_number(int index, float value)
{
  gmcal_data *data;
  int _error;
  
  if (!(data=gmcal_script_get_data_by_index(index))) {
    error("No matrix with index %d is in memory\n", (int) index);
    return 0;
  }

  if ((_error=gmcal_simple_matrix_divide_number(data->Z, data->dimX, data->dimY, value))) {
    warning("Error: %d\n", _error);
    return 0;
  }

  return data->index;
}

int gmcal_script_create_copy_with_normed_rows(int index)
{
  gmcal_data *data;
  gmcal_data *copy;
  int dimX;
  int dimY;
  float norm;
  int i, j;
  
  if (!(data=gmcal_script_get_data_by_index(index))) {
    error("No matrix with index %d is in memory\n", (int) index);
    return 0;
  }

  dimX=data->dimX;
  dimY=data->dimY;
  
  copy=gmcal_script_create_matrix(dimX, dimY);
  
  for (i=0; i<dimX; i++) {
     copy->X[i]=data->X[i];
  }
  for (i=0; i<dimY; i++) {
     copy->Y[i]=data->Y[i];
  }
  
  for (i=0; i<dimY; i++) {
    norm=data->Z[i][0];
    for (j=1; j<dimX; j++) {
       norm=max(norm,data->Z[i][j]);
    }
    for (j=0; j<dimX; j++) {
       copy->Z[i][j]=data->Z[i][j]/norm;
    }
  }
  
  return copy->index;
}

int gmcal_script_create_copy_with_normed_cols(int index)
{
  gmcal_data *data;
  gmcal_data *copy;
  int dimX;
  int dimY;
  float norm;
  int i, j;
  
  if (!(data=gmcal_script_get_data_by_index(index))) {
    error("No matrix with index %d is in memory\n", (int) index);
    return 0;
  }

  dimX=data->dimX;
  dimY=data->dimY;
  
  copy=gmcal_script_create_matrix(dimX, dimY);
  
  for (i=0; i<dimX; i++) {
     copy->X[i]=data->X[i];
  }
  for (i=0; i<dimY; i++) {
     copy->Y[i]=data->Y[i];
  }
  
  for (i=0; i<dimX; i++) {
    norm=data->Z[0][i];
    for (j=1; j<dimY; j++) {
       norm=max(norm,data->Z[j][i]);
    }
    for (j=0; j<dimY; j++) {
       copy->Z[j][i]=data->Z[j][i]/norm;
    }
  }
  
  return copy->index;
}

int gmcal_script_create_copy_with_normed_matrix(int index)
{
  gmcal_data *data;
  gmcal_data *copy;
  int dimX;
  int dimY;
  float norm;
  int i, j;
  
  if (!(data=gmcal_script_get_data_by_index(index))) {
    error("No matrix with index %d is in memory\n", (int) index);
    return 0;
  }

  dimX=data->dimX;
  dimY=data->dimY;
  
  copy=gmcal_script_create_matrix(dimX, dimY);
  
  for (i=0; i<dimX; i++) {
     copy->X[i]=data->X[i];
  }
  for (i=0; i<dimY; i++) {
     copy->Y[i]=data->Y[i];
  }
  
  norm=data->Z[0][0];
  for (i=0; i<dimY; i++) {
    for (j=i; j<dimX; j++) {
       norm=max(norm,data->Z[i][j]);
    }
  }

  for (i=0; i<dimY; i++) {
    for (j=0; j<dimX; j++) {
       copy->Z[i][j]=data->Z[i][j]/norm;
    }
  }
  
  return copy->index;
}

int gmcal_script_row_polynom(int index, int row, double a0, double a1, double a2, double a3, double a4, double a5)
{
  gmcal_data *data;
  int dimX;
  int dimY;
  float x;
  int i;
  
  if (!(data=gmcal_script_get_data_by_index(index))) {
    error("No matrix with index %d is in memory\n", (int) index);
    return 0;
  }

  dimX=data->dimX;
  dimY=data->dimY;
  
  if (row<0 || row>=dimY) {
    error("Row index (%d) must be between 0 and dimY-1 (%d); Nothing could be done\n", (int) row, dimY-1);
    return 0;
  }
  
  for (i=0; i<dimX; i++) {
    x=data->X[i];
    data->Z[row][i]=((((a5*x+a4)*x+a3)*x+a2)*x+a1)*x+a0;
  }

  return 1;
}

int gmcal_script_col_polynom(int index, int col, double a0, double a1, double a2, double a3, double a4, double a5)
{
  gmcal_data *data;
  int dimX;
  int dimY;
  float x;
  int i;
  
  if (!(data=gmcal_script_get_data_by_index(index))) {
    error("No matrix with index %d is in memory\n", (int) index);
    return 0;
  }

  dimX=data->dimX;
  dimY=data->dimY;
  
  if (col<0 || col>=dimX) {
    error("Col index (%d) must be between 0 and dimX-1 (%d)\nNothing could be done\n", (int) col, dimY-1);
    return 0;
  }
  
  for (i=0; i<dimY; i++) {
    x=data->Y[i];
    data->Z[i][col]=((((a5*x+a4)*x+a3)*x+a2)*x+a1)*x+a0;
  }

  return 1;
}

int gmcal_script_flip_matrix_rows(int index)
{
  gmcal_data *data;
  float dummy;
  int dimX;
  int dimY;
  int i, j;
  
  if (!(data=gmcal_script_get_data_by_index(index))) {
    error("No matrix with index %d is in memory\n", (int) index);
    return 0;
  }

  dimX=data->dimX;
  dimY=data->dimY;
  
  
  for (i=0; i<dimX/2; i++) {
    dummy=data->X[i];
    data->X[i]=data->X[dimX-i-1];
    data->X[dimX-i-1]=dummy;
  }
  for (i=0; i<dimY; i++) {
    for (j=0; j<dimX/2; j++) {
      dummy=data->Z[i][j];
      data->Z[i][j]=data->Z[i][dimX-j-1];
      data->Z[i][dimX-j-1]=dummy;
    }
  }

  return 1;
}

int gmcal_script_flip_matrix_cols(int index)
{
  gmcal_data *data;
  float dummy;
  int dimX;
  int dimY;
  int i, j;
  
  if (!(data=gmcal_script_get_data_by_index(index))) {
    error("No matrix with index %d is in memory\n", (int) index);
    return 0;
  }

  dimX=data->dimX;
  dimY=data->dimY;
  
  
  for (i=0; i<dimY/2; i++) {
    dummy=data->Y[i];
    data->Y[i]=data->Y[dimY-i-1];
    data->Y[dimY-i-1]=dummy;
  }
  for (i=0; i<dimX; i++) {
    for (j=0; j<dimY/2; j++) {
      dummy=data->Z[j][i];
      data->Z[j][i]=data->Z[dimX-j-1][i];
      data->Z[dimX-j-1][i]=dummy;
    }
  }

  return 1;
}

int gmcal_script_create_resized_matrix(int index, int newX, int newY)
{
  gmcal_data *data;
  gmcal_data *copy;
  gmcal_data *dummy;
  int dimX;
  int dimY;
  float min, max;
  int i, j;
  
  if (!(data=gmcal_script_get_data_by_index(index))) {
    error("No matrix with index %d is in memory\n", (int) index);
    return 0;
  }

  dimX=data->dimX;
  dimY=data->dimY;
  
  copy=gmcal_script_create_matrix(newX, newY);
  dummy=gmcal_script_create_matrix(dimY, newX);

  for (i=0; i<newX; i++) {
    min=max(0.,(i+0.)*dimX/newX);
    max=min(dimX-0.000001,(i+1.)*dimX/newX);
    copy->X[i]=gmcal_script_array_integral(data->X, min, max)/(max-min);
  }

  for (i=0; i<newY; i++) {
    min=max(0.,(i+0.)*dimY/newY);
    max=min(dimY-0.000001,(i+1.)*dimY/newY);
    copy->Y[i]=gmcal_script_array_integral(data->Y, min, max)/(max-min);
  }
  
  for (i=0; i<newX; i++) {
    min=max(0.,(i+0.)*dimX/newX);
    max=min(dimX-0.000001,(i+1.)*dimX/newX);
    for (j=0; j<dimY; j++) {
      dummy->Z[i][j]=gmcal_script_array_integral(data->Z[j], min, max)/(max-min);
    }
  }

  for (i=0; i<newY; i++) {
    min=max(0.,(i+0.)*dimY/newY);
    max=min(dimY-0.000001,(i+1.)*dimY/newY);
    for (j=0; j<newX; j++) {
      copy->Z[i][j]=gmcal_script_array_integral(dummy->Z[j], min, max)/(max-min);
    }
  }
  
  gmcal_script_delete_matrix(dummy);
  
  return copy->index;
}

int gmcal_script_create_matrix_cut(int index, int left, int top, int width, int height)
{
  gmcal_data *data;
  gmcal_data *copy;
  int i, j;
  
  if (!(data=gmcal_script_get_data_by_index(index))) {
    error("No matrix with index %d is in memory\n", (int) index);
    return 0;
  }

  if (width<1 || height<1) {
    error("Width and height must be greater than 0\n");
    return 0;
  }
  
  if (left+width>data->dimX) {
    error("Left+width must be equal to or less than dimX\n");
    return 0;
  }
  
  if (top+height>data->dimY) {
    error("Top+height must be equal to or less than dimY\n");
    return 0;
  }
  
  copy=gmcal_script_create_matrix(width, height);
  

//printf("<br>left: %d, top: %d, width: %d, height: %d\n<br>\n", left, top, width, height);

  for (i=0; i<width; i++) {
    for (j=0; j<height; j++) {
      copy->Z[j][i]=data->Z[j+top][i+left];
    }
  }
  for (i=0; i<width; i++) {
    copy->X[i]=data->X[i+left];
  }
  for (i=0; i<height; i++) {
    copy->Y[i]=data->Y[i+top];
  }
  

  
  return copy->index;
}

int gmcal_script_create_planck_from_x(int index, double temp)
{
  gmcal_data *data;
  gmcal_data *copy;
  double h=6.626176E-34;
  double c=2.99792458E8;
  double k=1.38066E-23;
  double value1, value2;
  double lambda;
  int dimX;
  int dimY;
  int i;
  
  if (!(data=gmcal_script_get_data_by_index(index))) {
    error("No matrix with index %d is in memory\n", (int) index);
    return 0;
  }

  dimX=data->dimX;
  dimY=1;
  
  copy=gmcal_script_create_matrix(dimX,dimY);

  for (i=0; i<dimX; i++) {
    copy->X[i]=data->X[i];
  }
  for (i=0; i<dimY; i++) {
    copy->Y[i]=0;
  }

  for (i=0; i<dimX; i++) {
    lambda=data->X[i]*1e-9;
    value1=(double) (2.*PI*h*c/(lambda*lambda*lambda*lambda*lambda));
    value2=(double) ( exp(h*c/(lambda*k*temp))-1. );
    copy->Z[0][i]=value1/value2;
  }

  return copy->index;
}

int gmcal_script_create_spline_from_x_and_row(int index, int row, float minX, float maxX, int steps)
{
  gmcal_data *data=NULL;
  gmcal_data *copy=NULL;
  double *coeff=NULL;
  int pos;
  int dimX;
  int dimY;
  int i;
  
  if (!(data=gmcal_script_get_data_by_index(index))) {
    error("No matrix with index %d is in memory\n", (int) index);
    return 0;
  }

  dimX=data->dimX;
  dimY=1;
  
  copy=gmcal_script_create_matrix(steps,dimY);

  for (i=0; i<steps; i++) {
    copy->X[i]=minX+i*(maxX-minX)/(steps-1);
  }
  for (i=0; i<dimY; i++) {
    copy->Y[i]=i;
  }

  gmcal_spline_get_coeff(data->X, data->Z[row], dimX, &coeff);

  for (i=0; i<steps; i++) {
    pos=gmcal_spline_get_interval(data->X, dimX, copy->X[i]);
    copy->Z[0][i]=gmcal_spline_get_value(data->X, data->Z[row], dimX, coeff, copy->X[i], pos);
  }

  return copy->index;
}

int gmcal_script_create_matrix_from_row(int index, int row, int count)
{
  gmcal_data *data;
  gmcal_data *copy;
  int dimX;
  int dimY;
  int i, j;
  
  if (!(data=gmcal_script_get_data_by_index(index))) {
    error("No matrix with index %d is in memory\n", (int) index);
    return 0;
  }

  dimX=data->dimX;
  dimY=count;
  
  copy=gmcal_script_create_matrix(dimX,dimY);

  for (i=0; i<dimX; i++) {
    copy->X[i]=data->X[i];
  }
  for (i=0; i<dimY; i++) {
    copy->Y[i]=i;
  }

  for (i=0; i<dimY; i++) {
    for (j=0; j<dimX; j++) {
      copy->Z[i][j]=data->Z[0][j];
    }
  }

  return copy->index;
}

int gmcal_script_set_dispersion_x_from_row(int target, int index, int row)
{
  gmcal_data *data;
  gmcal_data *copy;
  int dimX;
  int dimY;
  int i;
  
  if (!(data=gmcal_script_get_data_by_index(index))) {
    error("No matrix with index %d is in memory\n", (int) index);
    return 0;
  }

  dimX=data->dimX;
  dimY=data->dimY;
  
  if (!(copy=gmcal_script_get_data_by_index(target))) {
    error("No matrix with index %d is in memory\n", (int) index);
    return 0;
  }

  if (dimX!=copy->dimX) {
    error("Target %d and source %d do not have the same X-dimension\n", (int) target, (int) index);
    return 0;
  }

  if (dimY<=row) {
    error("Row index %d is greater than Y-dimension of matrix %d\n", (int) row, (int) index);
    return 0;
  }

  for (i=0; i<dimX; i++) {
    copy->X[i]=data->Z[row][i];
  }

  return 1;
}

