#include <glib.h>
#include <gfit_math.h>
#include <gfit_engine_profile.h>

gint
gfit_gauss(gdouble *target, gint num, gdouble **value)
/*----------------------------------------------------------------------
 *
 * value[0]: X
 * value[1]: X0		Peak position
 * value[2]: Y0		Peak value
 * value[3]: FWHM	Full width at half maximum
 *
 * Calculates the value of a Gaussian at position X.
 * 
 * ( 4*ln(2) = 2.7725887 )
 *
 *----------------------------------------------------------------------*/
{
  gdouble factor;
  
  if (num!=4) return 0;
      
  if (*value[2]!=0 && *value[3]>0 ) {
     factor=(*value[0]-*value[1])/(*value[3]);
     *target=*value[2]*exp(-2.7725887*factor*factor);
  } else {
    *target=0;
  }
  
  return 1;
}

gint
gfit_lorentz(gdouble *target, gint num, gdouble **value)
/*----------------------------------------------------------------------
 *
 * value[0]: X
 * value[1]: X0		Peak position
 * value[2]: Y0		Peak value
 * value[3]: FWHM	Full width at half maximum
 *
 * Calculates the value of a Lorentzian at position X.
 * 
 *----------------------------------------------------------------------*/
{
  gdouble factor;
  
  if (num!=4) return 0;
      
  if (*value[2]!=0 && *value[3]>0 ) {
     factor=2.0*(*value[0]-*value[1])/(*value[3]);
     *target=*value[2]/(1.0+factor*factor);
  } else {
    *target=0;
  }
        
  return 1;
}

static float T[10]={
      0.2453407083,
      0.7374737285,
      1.2340762153,
      1.7385377121,
      2.2549740020,
      2.7888060584,
      3.3478545673,
      3.9447640401,
      4.6036824495,
      5.3874808900
};

static float W[10]={
      4.6224366960E-1,
      2.8667550536E-1,
      1.0901720602E-1,
      2.4810520887E-2,
      3.2437733422E-3,
      2.2833863601E-4,
      7.8025564785E-6,
      1.0860693707E-7,
      4.3993409922E-10,
      2.2293936455E-13
};

static gdouble SLIM(gdouble X, gdouble Y);
static gdouble CLENSHAW(gdouble X, gdouble Y);
static gdouble SLIM2(gdouble X, gdouble Y);
static gdouble SLIM3(gdouble X, gdouble Y);
static gdouble Hui(gdouble X, gdouble Y);

gint
gfit_voigt(gdouble *target, gint num, gdouble **value)
/*----------------------------------------------------------------------
 *
 * value[0]: X
 * value[1]: X0			Peak position
 * value[2]: Y0			Peak value
 * value[3]: FWHMGauss		Full width at half maximum of Gaussian part
 * value[4]: FWHMLorentz	Full width at half maximum of Lorentzian Part
 *
 * Calculates the value of a Voigt-profil at position X.
 * 
 *----------------------------------------------------------------------*/
{
  gdouble D=ABS(*value[0]-*value[1]);
  gdouble Y;
  gdouble Z;
  
  if (num!=5) return 0;
  
  if (*value[2]==0) {
    *target=0;
    return 1;
  }
  if (*value[3]<=0) {
    gdouble *vals[]={value[0], value[1], value[2], value[4]};
    gfit_lorentz(target, 4, vals);
    return 1;
  } 
  if (*value[4]<=0) {
    gfit_gauss(target, 4, value);
    return 1;
  }
  
  Z=1.6651092222*D/(*value[3]);
  Y=0.8325546111*(*value[4])/(*value[3]);
  if (Z<0.001) {
    *target=*value[2];
  } else {
    if ((*value[3]>5.0) && (*value[4]>5.0)) {
      *target=*value[2]*Hui(Z,Y)/Hui(0.0,Y);
    } else {
      *target=*value[2]*SLIM(Z,Y)/SLIM(0.0,Y);
    }
  }
  
  return 1;
}

/*---------------------------------------------------------------------*/
static gdouble
SLIM(gdouble X, gdouble Y)
{
  gdouble Y2,Y300; 
      
  Y2=Y*Y;
  Y300=1.8/(1.0+X); 
  if (((Y<1.0) && (X<4.0)) || (Y<Y300)) {
    return(CLENSHAW(X,Y));
  } else {
    if ((Y<2.5) && (X<4.0)) {
      return(SLIM2(X,Y));
    } else {
      return(SLIM3(X,Y));
    }
  }
}

static gdouble
CLENSHAW(gdouble X, gdouble Y)
{
  gdouble X2=X*X,Y2=Y*Y,U1=0.0,EU1,BN01=0.0,BN02=0.0,BN,X1=X/5.0,F;
  gdouble COEF=4.0*X1*X1-2.0,DN,DN01,DN02,FUNCT,Q=1.0,G;
  gdouble wert;
  int i;
  gdouble C[34]={
    0.1999999999972224,
    -.1840000000029998,
    0.1558399999965025,
    -.1216640000043988,
    0.0877081599940391,
    -.0585141248086907,
    0.0362157301623914,
    -.0208497654398036,
    0.0111960116346270,
    -.56231896167109E-2,
    0.26487634172265E-2,
    -.11732670757704E-2,
    0.4899519978088E-3,
    -.193363080152E-3,
    0.722877446788E-4,
    -.256555125979E-4,
    0.86620736841E-5,
    -.27876379719E-5,
    0.8566873627E-6,
    -.2518433784E-6,
    0.709360221E-7,
    -.191732257E-7,
    0.49801256E-8,
    -.12447734E-8,
    0.2997777E-9,
    -.696450E-10,
    0.156262E-10,
    -.33897E-11,
    0.7116E-12,
    -.1447E-12,
    0.285E-13,
    -.55E-14,
    0.10E-14,
    -.2E-15
  };

  if ((-(EU1=Y2-X2)<=70.0) && ((170.0-EU1)>0.0) && ((170.0+EU1)>0.0)) 
      U1=exp(EU1)*cos(2.0*X*Y);
  if (X<=5.0) {

/* CLENSHAWS ALGORITHM AS GIVEN BY HUMMER */

    for (i=33; i>-1; i--) {
      BN=COEF*BN01-BN02+C[i];
      BN02=BN01;
      BN01=BN;
    }
    F=X1*(BN-BN02);
    DN01=1.0-2.0*X*F;
    DN02=F;
  } else {
    DN01=-((((((1055.7421/X2+162.4218)/X2+29.53125)/X2+6.5625)/X2+1.875)/X2+
            0.75)/X2+0.5)/X2;
    DN02=(1.0-DN01)/(2.0*X);
  }
  FUNCT=Y*DN01;
  if (Y>1.0E-8) {
  for (i=2; i<51;++i) {
    DN=-(X*DN01+DN02)*2.0/i;
    DN02=DN01;
    DN01=DN;
    if (i%2) {
      Q=-Q;
      Y*=Y2;
      G=DN*Y;
      FUNCT+=Q*G;
      if (((G/FUNCT)<=1.0E-8) && ((G/FUNCT)>=-1.0E-8)) break; 
      }
    }
  }
  wert=U1-1.12837917*FUNCT;
  return(wert);
}


/*--------------------------------------------------------------------*/
static gdouble
SLIM2(gdouble X, gdouble Y)
{     
  int i;
  gdouble G=0.0,R,S,Y2=Y*Y;
  gdouble wert;
  for (i=0; i<10; i++) {
    R=T[i]-X;
    S=T[i]+X;
    G=G+(4.0*T[i]*T[i]-2.0)*(R*atan(R/Y)+S*atan(S/Y)-0.5*Y*
        (log(Y2+R*R)+log(Y2+S*S)))*W[i];
    }
  wert=0.318309886*G;
  return(wert);
}
/*---------------------------------------------------------------------*/
  
static gdouble
SLIM3(gdouble X, gdouble Y)
{     
  gdouble G=0.0,Y2=Y*Y;
  int i;

  for (i=0; i<10; i++)
    G=G+(1.0/((X-T[i])*(X-T[i])+Y2)+1.0/((X+T[i])*(X+T[i])+Y2))*W[i];
  return(0.318309886*Y*G);
}


/*---------------------------------------------------------------------*/
static gdouble
Hui(gdouble X, gdouble Y)
/*  6th degree rational complex approximation by Hui, Armstrong, Wray  */
/*  Umgeschrieben auf reelle Berechnung von R.Bock                '94  */
/*---------------------------------------------------------------------*/
{
  
  gdouble A0,A1,A2,A3,A4,A5,A6,B0,B1,B2,B3,B4,B5,B6;
  gdouble R1,R2,R3,R4,R5,R6,R7,I1,I2,I3,I4,I5,I6,I7;
  gdouble RZaehler,IZaehler,RNenner,INenner;
  gdouble wert;
  
  A0=122.60793178;
  A1=214.38238869;
  A2=181.92853309;
  A3=93.155580458;
  A4=30.180142196;
  A5=5.9126262098;
  A6=.56418958356;
  B0=122.60793178;
  B1=352.73062511;
  B2=457.33447878;
  B3=348.70391772;
  B4=170.35400182;
  B5=53.992906913;
  B6=10.479857114;
  
  R1=Y; 		I1=-X;
  R2=R1*R1-I1*I1;	I2=R1*I1+R1*I1;
  R3=R2*R1-I2*I1;	I3=R2*I1+R1*I2;
  R4=R3*R1-I3*I1;	I4=R3*I1+R1*I3;
  R5=R4*R1-I4*I1;	I5=R4*I1+R1*I4;
  R6=R5*R1-I5*I1;	I6=R5*I1+R1*I5;
  R7=R6*R1-I6*I1;	I7=R6*I1+R1*I6;
  
  RZaehler=A0+A1*R1+A2*R2+A3*R3+A4*R4+A5*R5+A6*R6;
  IZaehler=   A1*I1+A2*I2+A3*I3+A4*I4+A5*I5+A6*I6;
  RNenner =B0+B1*R1+B2*R2+B3*R3+B4*R4+B5*R5+B6*R6+R7;
  INenner =   B1*I1+B2*I2+B3*I3+B4*I4+B5*I5+B6*I6+I7;

  wert=(RZaehler*RNenner+IZaehler*INenner)/(RNenner*RNenner+INenner*INenner);
  

  return(wert);
}
